@@ -3,7 +3,7 @@
  +--------------------------------------------------------------------+
  | CiviCRM version 4.7                                                |
  +--------------------------------------------------------------------+
- | Copyright CiviCRM LLC (c) 2004-2016                                |
+ | Copyright CiviCRM LLC (c) 2004-2017                                |
  +--------------------------------------------------------------------+
  | This file is a part of CiviCRM.                                    |
  |                                                                    |
@@ -28,7 +28,7 @@
 /**
  *
  * @package CRM
- * @copyright CiviCRM LLC (c) 2004-2016
+ * @copyright CiviCRM LLC (c) 2004-2017
  */
 class CRM_Case_XMLProcessor_Process extends CRM_Case_XMLProcessor {
   /**
@@ -44,7 +44,7 @@ class CRM_Case_XMLProcessor_Process extends CRM_Case_XMLProcessor {
     $xml = $this->retrieve($caseType);
 
     if ($xml === FALSE) {
-      $docLink = CRM_Utils_System::docURL2("user/case-management/setup");
+      $docLink = CRM_Utils_System::docURL2("user/case-management/set-up");
       CRM_Core_Error::fatal(ts("Configuration file could not be retrieved for case type = '%1' %2.",
         array(1 => $caseType, 2 => $docLink)
       ));
@@ -69,7 +69,7 @@ class CRM_Case_XMLProcessor_Process extends CRM_Case_XMLProcessor {
   public function get($caseType, $fieldSet, $isLabel = FALSE, $maskAction = FALSE) {
     $xml = $this->retrieve($caseType);
     if ($xml === FALSE) {
-      $docLink = CRM_Utils_System::docURL2("user/case-management/setup");
+      $docLink = CRM_Utils_System::docURL2("user/case-management/set-up");
       CRM_Core_Error::fatal(ts("Unable to load configuration file for the referenced case type: '%1' %2.",
         array(1 => $caseType, 2 => $docLink)
       ));
@@ -217,7 +217,7 @@ class CRM_Case_XMLProcessor_Process extends CRM_Case_XMLProcessor {
     $relationshipTypeID = array_search($relationshipTypeName, $relationshipTypes);
 
     if ($relationshipTypeID === FALSE) {
-      $docLink = CRM_Utils_System::docURL2("user/case-management/setup");
+      $docLink = CRM_Utils_System::docURL2("user/case-management/set-up");
       CRM_Core_Error::fatal(ts('Relationship type %1, found in case configuration file, is not present in the database %2',
         array(1 => $relationshipTypeName, 2 => $docLink)
       ));
@@ -421,7 +421,7 @@ AND        a.is_deleted = 0
     $activityTypeInfo = CRM_Utils_Array::value($activityTypeName, $activityTypes);
 
     if (!$activityTypeInfo) {
-      $docLink = CRM_Utils_System::docURL2("user/case-management/setup");
+      $docLink = CRM_Utils_System::docURL2("user/case-management/set-up");
       CRM_Core_Error::fatal(ts('Activity type %1, found in case configuration file, is not present in the database %2',
         array(1 => $activityTypeName, 2 => $docLink)
       ));
@@ -437,12 +437,7 @@ AND        a.is_deleted = 0
       $statusName = 'Scheduled';
     }
 
-    if ($this->_isMultiClient) {
-      $client = $params['clientID'];
-    }
-    else {
-      $client = array(1 => $params['clientID']);
-    }
+    $client = (array) $params['clientID'];
 
     //set order
     $orderVal = '';
@@ -457,10 +452,7 @@ AND        a.is_deleted = 0
         'is_auto' => FALSE,
         'is_current_revision' => 1,
         'subject' => CRM_Utils_Array::value('subject', $params) ? $params['subject'] : $activityTypeName,
-        'status_id' => CRM_Core_OptionGroup::getValue('activity_status',
-          $statusName,
-          'name'
-        ),
+        'status_id' => CRM_Core_PseudoConstant::getKey('CRM_Activity_BAO_Activity', 'activity_status_id', $statusName),
         'target_contact_id' => $client,
         'medium_id' => CRM_Utils_Array::value('medium_id', $params),
         'location' => CRM_Utils_Array::value('location', $params),
@@ -475,10 +467,7 @@ AND        a.is_deleted = 0
         'source_contact_id' => $params['creatorID'],
         'is_auto' => TRUE,
         'is_current_revision' => 1,
-        'status_id' => CRM_Core_OptionGroup::getValue('activity_status',
-          $statusName,
-          'name'
-        ),
+        'status_id' => CRM_Core_PseudoConstant::getKey('CRM_Activity_BAO_Activity', 'activity_status_id', $statusName),
         'target_contact_id' => $client,
         'weight' => $orderVal,
       );
@@ -561,6 +550,7 @@ AND        a.is_deleted = 0
       $activityParams['skipRecentView'] = TRUE;
     }
 
+    // @todo - switch to using api & remove the parameter pre-wrangling above.
     $activity = CRM_Activity_BAO_Activity::create($activityParams);
 
     if (!$activity) {
