@@ -3,7 +3,7 @@
  +--------------------------------------------------------------------+
  | CiviCRM version 4.7                                                |
  +--------------------------------------------------------------------+
- | Copyright CiviCRM LLC (c) 2004-2016                                |
+ | Copyright CiviCRM LLC (c) 2004-2017                                |
  +--------------------------------------------------------------------+
  | This file is a part of CiviCRM.                                    |
  |                                                                    |
@@ -28,7 +28,7 @@
 /**
  *
  * @package CRM
- * @copyright CiviCRM LLC (c) 2004-2016
+ * @copyright CiviCRM LLC (c) 2004-2017
  */
 class CRM_Contact_Page_DedupeFind extends CRM_Core_Page_Basic {
   protected $_cid = NULL;
@@ -62,27 +62,30 @@ class CRM_Contact_Page_DedupeFind extends CRM_Core_Page_Basic {
     $context = CRM_Utils_Request::retrieve('context', 'String', $this);
     $limit = CRM_Utils_Request::retrieve('limit', 'Integer', $this);
     $rgid = CRM_Utils_Request::retrieve('rgid', 'Positive');
-    $urlQry = "reset=1&rgid={$rgid}&gid={$gid}&limit={$limit}";
-    $this->assign('urlQuery', $urlQry);
+    $urlQry = array(
+      'reset' => 1,
+      'rgid' => $rgid,
+      'gid' => $gid,
+      'limit' => $limit,
+    );
+    $this->assign('urlQuery', CRM_Utils_System::makeQueryString($urlQry));
 
-    $session = CRM_Core_Session::singleton();
-    $contactIds = $session->get('selectedSearchContactIds');
-    if ($context == 'search' || !empty($contactIds)) {
+    if ($context == 'search') {
       $context = 'search';
-      $this->assign('backURL', $session->readUserContext());
+      $this->assign('backURL', CRM_Core_Session::singleton()->readUserContext());
     }
 
     if ($action & CRM_Core_Action::RENEW) {
       // empty cache
-
       if ($rgid) {
         CRM_Core_BAO_PrevNextCache::deleteItem(NULL, CRM_Dedupe_Merger::getMergeCacheKeyString($rgid, $gid));
       }
-      CRM_Utils_System::redirect(CRM_Utils_System::url('civicrm/contact/dedupefind', $urlQry . "&action=update"));
+      $urlQry['action'] = 'update';
+      CRM_Utils_System::redirect(CRM_Utils_System::url('civicrm/contact/dedupefind', $urlQry));
     }
     elseif ($action & CRM_Core_Action::MAP) {
       // do a batch merge if requested
-      $result = CRM_Dedupe_Merger::batchMerge($rgid, $gid, 'safe', TRUE, 75);
+      $result = CRM_Dedupe_Merger::batchMerge($rgid, $gid, 'safe', 75);
 
       $skippedCount = CRM_Utils_Request::retrieve('skipped', 'Positive', $this, FALSE, 0);
       $skippedCount = $skippedCount + count($result['skipped']);
@@ -102,10 +105,13 @@ class CRM_Contact_Page_DedupeFind extends CRM_Core_Page_Basic {
         }
         $message .= ts(" during the batch merge process with safe mode.");
         CRM_Core_Session::setStatus($message, ts('Merge Complete'), 'success');
-        CRM_Utils_System::redirect(CRM_Utils_System::url('civicrm/contact/dedupefind', $urlQry . "&action=update"));
+        $urlQry['action'] = 'update';
+        CRM_Utils_System::redirect(CRM_Utils_System::url('civicrm/contact/dedupefind', $urlQry));
       }
       else {
-        $urlQry .= "&action=map&skipped={$skippedCount}&merged={$mergedCount}";
+        $urlQry['action'] = 'map';
+        $urlQry['skipped'] = $skippedCount;
+        $urlQry['merged'] = $mergedCount;
         CRM_Utils_System::jsRedirect(
           CRM_Utils_System::url('civicrm/contact/dedupefind', $urlQry),
           ts('Batch Merge Task in progress'),
@@ -120,9 +126,9 @@ class CRM_Contact_Page_DedupeFind extends CRM_Core_Page_Basic {
       $cid = CRM_Utils_Request::retrieve('cid', 'Positive', $this, FALSE, 0);
       $this->action = CRM_Core_Action::UPDATE;
 
-      $urlQry .= '&snippet=4';
+      $urlQry['snippet'] = 4;
       if ($context == 'conflicts') {
-        $urlQry .= "&selected=1";
+        $urlQry['selected'] = 1;
       }
 
       $this->assign('sourceUrl', CRM_Utils_System::url('civicrm/ajax/dedupefind', $urlQry, FALSE, NULL, FALSE));
@@ -145,7 +151,9 @@ class CRM_Contact_Page_DedupeFind extends CRM_Core_Page_Basic {
       if (empty($this->_mainContacts)) {
         if ($context == 'conflicts') {
           // if the current screen was intended to list only selected contacts, move back to full dupe list
-          CRM_Utils_System::redirect(CRM_Utils_System::url(CRM_Utils_System::currentPath(), $urlQry . '&action=update'));
+          $urlQry['action'] = 'update';
+          unset($urlQry['snippet']);
+          CRM_Utils_System::redirect(CRM_Utils_System::url(CRM_Utils_System::currentPath(), $urlQry));
         }
         if ($gid) {
           $foundDupes = $this->get("dedupe_dupes_$gid");
@@ -154,13 +162,6 @@ class CRM_Contact_Page_DedupeFind extends CRM_Core_Page_Basic {
           }
           $this->set("dedupe_dupes_$gid", $foundDupes);
         }
-        elseif (!empty($contactIds)) {
-          $foundDupes = $this->get("search_dedupe_dupes_$gid");
-          if (!$foundDupes) {
-            $foundDupes = CRM_Dedupe_Finder::dupes($rgid, $contactIds);
-          }
-          $this->set("search_dedupe_dupes_$gid", $foundDupes);
-        }
         else {
           $foundDupes = $this->get('dedupe_dupes');
           if (!$foundDupes) {
@@ -193,15 +194,16 @@ class CRM_Contact_Page_DedupeFind extends CRM_Core_Page_Basic {
           $this->_rgid = $rgid;
           $this->_mainContacts = $mainContacts;
 
-          $session = CRM_Core_Session::singleton();
+          $urlQry['action'] = 'update';
           if ($this->_cid) {
-            $session->pushUserContext(CRM_Utils_System::url('civicrm/contact/deduperules',
-              $urlQry . "&action=update&cid={$this->_cid}"
+            $urlQry['cid'] = $this->_cid;
+            CRM_Core_Session::singleton()->pushUserContext(CRM_Utils_System::url('civicrm/contact/deduperules',
+              $urlQry
             ));
           }
           else {
-            $session->pushUserContext(CRM_Utils_System::url('civicrm/contact/dedupefind',
-              $urlQry . "&action=update"
+            CRM_Core_Session::singleton()->pushUserContext(CRM_Utils_System::url('civicrm/contact/dedupefind',
+              $urlQry
             ));
           }
         }
@@ -226,7 +228,6 @@ class CRM_Contact_Page_DedupeFind extends CRM_Core_Page_Basic {
     }
     $this->assign('context', $context);
 
-    // parent run
     return parent::run();
   }
 
