@@ -602,6 +602,14 @@ class CRM_Core_BAO_Mapping extends CRM_Core_DAO_Mapping {
                 elseif (isset($relationshipType->$target_type)) {
                   $relatedFields = array_merge((array) $relatedMapperFields[$relationshipType->$target_type], (array) $relationshipCustomFields);
                 }
+                //CRM-20672 If contact target type not set e.g. "All Contacts" relationship - present user with all field options and let them determine what they expect to work
+                else {
+                  $types = CRM_Contact_BAO_ContactType::basicTypes(FALSE);
+                  foreach ($types as $contactType => $label) {
+                    $relatedFields = array_merge($relatedFields, (array) $relatedMapperFields[$label]);
+                  }
+                  $relatedFields = array_merge($relatedFields, (array) $relationshipCustomFields);
+                }
               }
               $relationshipType->free();
               asort($relatedFields);
