@@ -1,38 +1,38 @@
 <?php
 /*
- +--------------------------------------------------------------------+
- | CiviCRM version 4.7                                                |
- +--------------------------------------------------------------------+
- | Copyright CiviCRM LLC (c) 2004-2016                                |
- +--------------------------------------------------------------------+
- | This file is a part of CiviCRM.                                    |
- |                                                                    |
- | CiviCRM is free software; you can copy, modify, and distribute it  |
- | under the terms of the GNU Affero General Public License           |
- | Version 3, 19 November 2007 and the CiviCRM Licensing Exception.   |
- |                                                                    |
- | CiviCRM is distributed in the hope that it will be useful, but     |
- | WITHOUT ANY WARRANTY; without even the implied warranty of         |
- | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.               |
- | See the GNU Affero General Public License for more details.        |
- |                                                                    |
- | You should have received a copy of the GNU Affero General Public   |
- | License and the CiviCRM Licensing Exception along                  |
- | with this program; if not, contact CiviCRM LLC                     |
- | at info[AT]civicrm[DOT]org. If you have questions about the        |
- | GNU Affero General Public License or the licensing of CiviCRM,     |
- | see the CiviCRM license FAQ at http://civicrm.org/licensing        |
- +--------------------------------------------------------------------+
++--------------------------------------------------------------------+
+| CiviCRM version 4.7                                                |
++--------------------------------------------------------------------+
+| Copyright CiviCRM LLC (c) 2004-2017                                |
++--------------------------------------------------------------------+
+| This file is a part of CiviCRM.                                    |
+|                                                                    |
+| CiviCRM is free software; you can copy, modify, and distribute it  |
+| under the terms of the GNU Affero General Public License           |
+| Version 3, 19 November 2007 and the CiviCRM Licensing Exception.   |
+|                                                                    |
+| CiviCRM is distributed in the hope that it will be useful, but     |
+| WITHOUT ANY WARRANTY; without even the implied warranty of         |
+| MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.               |
+| See the GNU Affero General Public License for more details.        |
+|                                                                    |
+| You should have received a copy of the GNU Affero General Public   |
+| License and the CiviCRM Licensing Exception along                  |
+| with this program; if not, contact CiviCRM LLC                     |
+| at info[AT]civicrm[DOT]org. If you have questions about the        |
+| GNU Affero General Public License or the licensing of CiviCRM,     |
+| see the CiviCRM license FAQ at http://civicrm.org/licensing        |
++--------------------------------------------------------------------+
  */
 
 /**
  *
  * @package CRM
- * @copyright CiviCRM LLC (c) 2004-2016
+ * @copyright CiviCRM LLC (c) 2004-2017
  * $Id$
  *
  */
-class CRM_Core_I18n_SchemaStructure_3_0_beta4 {
+class CRM_Core_I18n_SchemaStructure_3_2_beta4 {
   /**
    * @return array
    */
@@ -44,11 +44,6 @@ class CRM_Core_I18n_SchemaStructure_3_0_beta4 {
           'label' => 'varchar(255)',
           'description' => 'varchar(255)',
         ),
-        'civicrm_price_set' => array(
-          'title' => 'varchar(255)',
-          'help_pre' => 'text',
-          'help_post' => 'text',
-        ),
         'civicrm_contact' => array(
           'sort_name' => 'varchar(128)',
           'display_name' => 'varchar(128)',
@@ -61,6 +56,10 @@ class CRM_Core_I18n_SchemaStructure_3_0_beta4 {
           'household_name' => 'varchar(128)',
           'organization_name' => 'varchar(128)',
         ),
+        'civicrm_contact_type' => array(
+          'label' => 'varchar(64)',
+          'description' => 'text',
+        ),
         'civicrm_mailing_component' => array(
           'name' => 'varchar(64)',
           'subject' => 'varchar(255)',
@@ -88,7 +87,7 @@ class CRM_Core_I18n_SchemaStructure_3_0_beta4 {
           'description' => 'varchar(255)',
         ),
         'civicrm_membership_status' => array(
-          'name' => 'varchar(128)',
+          'label' => 'varchar(128)',
         ),
         'civicrm_participant_status_type' => array(
           'label' => 'varchar(255)',
@@ -100,6 +99,11 @@ class CRM_Core_I18n_SchemaStructure_3_0_beta4 {
           'thankyou_title' => 'varchar(255)',
           'thankyou_text' => 'text',
         ),
+        'civicrm_price_set' => array(
+          'title' => 'varchar(255)',
+          'help_pre' => 'text',
+          'help_post' => 'text',
+        ),
         'civicrm_custom_group' => array(
           'title' => 'varchar(64)',
           'help_pre' => 'text',
@@ -110,15 +114,13 @@ class CRM_Core_I18n_SchemaStructure_3_0_beta4 {
           'help_pre' => 'text',
           'help_post' => 'text',
         ),
+        'civicrm_dashboard' => array(
+          'label' => 'varchar(255)',
+        ),
         'civicrm_option_value' => array(
           'label' => 'varchar(255)',
           'description' => 'varchar(255)',
         ),
-        'civicrm_price_field' => array(
-          'label' => 'varchar(255)',
-          'help_pre' => 'text',
-          'help_post' => 'text',
-        ),
         'civicrm_contribution_page' => array(
           'title' => 'varchar(255)',
           'intro_text' => 'text',
@@ -140,6 +142,11 @@ class CRM_Core_I18n_SchemaStructure_3_0_beta4 {
           'renewal_title' => 'varchar(255)',
           'renewal_text' => 'text',
         ),
+        'civicrm_price_field' => array(
+          'label' => 'varchar(255)',
+          'help_pre' => 'text',
+          'help_post' => 'text',
+        ),
         'civicrm_uf_group' => array(
           'title' => 'varchar(64)',
           'help_pre' => 'text',
@@ -147,6 +154,7 @@ class CRM_Core_I18n_SchemaStructure_3_0_beta4 {
         ),
         'civicrm_uf_field' => array(
           'help_post' => 'text',
+          'help_pre' => 'text',
           'label' => 'varchar(255)',
         ),
         'civicrm_address' => array(
@@ -192,15 +200,6 @@ class CRM_Core_I18n_SchemaStructure_3_0_beta4 {
     static $result = NULL;
     if (!$result) {
       $result = array(
-        'civicrm_price_set' => array(
-          'UI_title' => array(
-            'name' => 'UI_title',
-            'field' => array(
-              'title',
-            ),
-            'unique' => 1,
-          ),
-        ),
         'civicrm_contact' => array(
           'index_sort_name' => array(
             'name' => 'index_sort_name',
@@ -233,6 +232,15 @@ class CRM_Core_I18n_SchemaStructure_3_0_beta4 {
             ),
           ),
         ),
+        'civicrm_price_set' => array(
+          'UI_title' => array(
+            'name' => 'UI_title',
+            'field' => array(
+              'title',
+            ),
+            'unique' => 1,
+          ),
+        ),
         'civicrm_custom_group' => array(
           'UI_title_extends' => array(
             'name' => 'UI_title_extends',
