@@ -3,7 +3,7 @@
  +--------------------------------------------------------------------+
  | CiviCRM version 4.7                                                |
  +--------------------------------------------------------------------+
- | Copyright CiviCRM LLC (c) 2004-2016                                |
+ | Copyright CiviCRM LLC (c) 2004-2017                                |
  +--------------------------------------------------------------------+
  | This file is a part of CiviCRM.                                    |
  |                                                                    |
@@ -28,7 +28,7 @@
 /**
  *
  * @package CRM
- * @copyright CiviCRM LLC (c) 2004-2016
+ * @copyright CiviCRM LLC (c) 2004-2017
  */
 
 /**
@@ -162,8 +162,8 @@ class CRM_Utils_System_Drupal extends CRM_Utils_System_DrupalBase {
    * @inheritDoc
    */
   public function getLoginURL($destination = '') {
-    $query = $destination ? array('destination' => $destination) : array();
-    return url('user', array('query' => $query), TRUE);
+    $query = $destination ? array('destination' => $destination) : NULL;
+    return CRM_Utils_System::url('user', $query, TRUE);
   }
 
   /**
@@ -299,6 +299,11 @@ class CRM_Utils_System_Drupal extends CRM_Utils_System_DrupalBase {
     $base_url = str_replace('http://', 'https://', $base_url);
   }
 
+  /**
+   * Get the name of the users table.
+   *
+   * @return string
+   */
   protected function getUsersTableName() {
     $userFrameworkUsersTableName = Civi::settings()->get('userFrameworkUsersTableName');
     if (empty($userFrameworkUsersTableName)) {
@@ -349,6 +354,8 @@ class CRM_Utils_System_Drupal extends CRM_Utils_System_DrupalBase {
       $strtolower = function_exists('mb_strtolower') ? 'mb_strtolower' : 'strtolower';
       $name = $dbDrupal->escapeSimple($strtolower($name));
       $userFrameworkUsersTableName = $this->getUsersTableName();
+
+      // LOWER in query below roughly translates to 'hurt my database without deriving any benefit' See CRM-19811.
       $sql = "
 SELECT u.*
 FROM   {$userFrameworkUsersTableName} u
@@ -467,8 +474,7 @@ AND    u.status = 1
 
     if (!file_exists("$cmsPath/includes/bootstrap.inc")) {
       if ($throwError) {
-        echo '<br />Sorry, could not locate bootstrap.inc\n';
-        exit();
+        throw new Exception('Sorry, could not locate bootstrap.inc');
       }
       return FALSE;
     }
@@ -488,11 +494,18 @@ AND    u.status = 1
     @drupal_bootstrap(DRUPAL_BOOTSTRAP_FULL);
 
     // explicitly setting error reporting, since we cannot handle drupal related notices
+    // @todo 1 = E_ERROR, but more to the point setting error reporting deep in code
+    // causes grief with debugging scripts
     error_reporting(1);
-    if (!function_exists('module_exists') || !module_exists('civicrm')) {
+    if (!function_exists('module_exists')) {
       if ($throwError) {
-        echo '<br />Sorry, could not load drupal bootstrap.';
-        exit();
+        throw new Exception('Sorry, could not load drupal bootstrap.');
+      }
+      return FALSE;
+    }
+    if (!module_exists('civicrm')) {
+      if ($throwError) {
+        throw new Exception('Sorry, drupal cannot find CiviCRM');
       }
       return FALSE;
     }
@@ -524,8 +537,7 @@ AND    u.status = 1
         $uid = user_authenticate($name, $pass);
         if (!$uid) {
           if ($throwError) {
-            echo '<br />Sorry, unrecognized username or password.';
-            exit();
+            throw new Exception('Sorry, unrecognized username or password.');
           }
           return FALSE;
         }
@@ -542,8 +554,7 @@ AND    u.status = 1
     }
 
     if ($throwError) {
-      echo '<br />Sorry, can not load CMS user account.';
-      exit();
+      throw new Exception('Sorry, can not load CMS user account.');
     }
 
     // CRM-6948: When using loadBootStrap, it's implicit that CiviCRM has already loaded its settings
